#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2008 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>

import _shellfolders, util
import os, subprocess, string, random

import logging

class Bottles:
	def __init__(self):
		self._shellfolders = _shellfolders.ShellFolders()
		self.logging = logging.getLogger("python-wine.bottles")
		self.debug = self.logging.debug
		self.error = self.logging.error
	
	def list(self, includedir=False):
		HOME = util.getRealHome()
		if not os.path.isdir("%s/.winebottles" % HOME):
			os.mkdir("%s/.winebottles" % HOME)
			return []
		bottles = os.listdir("%s/.winebottles" % HOME)
		if includedir:
			bottlenames = {}
		else:
			bottlenames = []
		for bottle in bottles:
			if os.path.exists("%s/.winebottles/%s/.name" % (HOME, bottle)) and os.path.exists("%s/.winebottles/%s/.wine" % (HOME, bottle)):
				bottlename = self.getbottlename(bottle)
				if includedir:
					bottlenames[bottlename] = bottle
				else:
					bottlenames.append(bottlename)
		return bottlenames
	
	def add(self, bottlename):
		if not len(bottlename.strip()):
			raise ValueError, bottlename
			return
		HOME = util.getRealHome()
		bottledir = self.getdirname(bottlename)
		if bottledir:
			path = "%s/.winebottles/%s" % ( HOME, bottledir )
		else:
			if not os.path.isdir("%s/.winebottles" % HOME):
				os.mkdir("%s/.winebottles" % HOME)
			bottledir = self.createdirname()
			path = "%s/.winebottles/%s" % ( HOME, bottledir )
			if not os.path.isdir(path):
				os.mkdir(path)
			namefile = open("%s/.name" % path, 'w')
			namefile.write(bottlename)
			namefile.close()
		env = os.environ.copy()
		os.environ['HOME'] = path
		os.environ['WINEPREFIX'] = "%s/.wine" % path
		# Create Wine structure (in place of wineprefixcreate, since it's depricated)
		process = subprocess.Popen(["regedit", "/E", "/tmp/.winebottlecreated", 'HKEY_CURRENT_USER\\Software\\Wine'], stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=os.environ)
		process.wait()
		
		self._shellfolders.automap()
		
		return process.returncode == 0
	
	def remove(self, bottlename):
		HOME = util.getRealHome()
		path = "%s/.winebottles/%s" % ( HOME, self.getdirname(bottlename) )
		if os.path.isdir(path):
			process = subprocess.Popen(["rm", "-rf", path], stdout=subprocess.PIPE, env=os.environ).wait()
			return process == 0
		else:
			self.error("Bottle doesn't exist.")
			return True
	
	def use(self, bottlename):
		HOME = util.getRealHome()
		if bottlename == None:
			os.environ['WINEPREFIX'] = "%s/.wine" % HOME
			os.environ['HOME'] = "%s" % HOME
		else:
			bottledir = self.getdirname(bottlename)
			if bottledir:
				os.environ['WINEPREFIX'] = "%s/.winebottles/%s/.wine" % ( HOME, bottledir )
				os.environ['HOME'] = "%s/.winebottles/%s" % ( HOME, bottledir )
				return True
			else:
				return False
	
	def createdirname(self):
		HOME = util.getRealHome()
		
		def generatename():
			chars = string.ascii_lowercase+string.digits
			uniquestring = ''.join([ chars[random.randrange(len(chars))] for i in range(8) ])
			return 'bottle-%s' % uniquestring
		
		dirname = generatename()
		if os.path.exists("%s/.winebottles/%s" % ( HOME, dirname )):
			while os.path.exists("%s/.winebottles/%s" % ( HOME, dirname )):
				dirname = generatename()
		
		return dirname
	
	def getdirname(self, bottlename):
		bottles = self.list(True)
		for key in bottles.keys():
			bottles[key.lower()] = bottles[key]
		
		if bottlename.lower().strip() in bottles:
			return bottles[bottlename.lower().strip()]
		else:
			return False
	
	def getbottlename(self, dirname):
		HOME = util.getRealHome()
		if os.path.isdir("%s/.winebottles/%s" % (HOME, dirname)):
			namefile = open("%s/.winebottles/%s/.name" % (HOME, dirname), 'r')
			bottlename = namefile.read().strip()
			namefile.close()
			return bottlename
		else:
			return None
	
	def setbottlename(self, dirname, bottlename):
		HOME = util.getRealHome()
		if os.path.isdir("%s/.winebottles/%s" % (HOME, dirname)):
			try:
				namefile = open("%s/.winebottles/%s/.name" % (HOME, dirname), 'w')
			except IOError:
				print dirname
			namefile.write(bottlename.strip())
			namefile.close()
			return self.getbottlename(dirname)
		else:
			return None
