#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2008 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>

import _registry
import subprocess

registry = _registry.Registry()

class Graphics:
	def getScreenDPI(self):
		settings = registry.get('HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Hardware Profiles\\Current\\Software\\Fonts', quiet=True)
		if "LowPixels" in settings:
			return int(settings["LowPixels"], 16)
		else:
			return 96
		
	def setScreenDPI(self, value):
		dword = 'dword:000000'+hex(int(value))[2:]
		registry.set({'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Hardware Profiles\\Current\\Software\\Fonts': {"LogPixels": dword}})
	
	def getVertexShader(self, program=None):
		if program:
			settings = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\Direct3D' % program, quiet=True)
		else:
			settings = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\Direct3D', quiet=True)
		if "VertexShaderMode" in settings:
			return False
		else:
			return True
	
	def setVertexShader(self, value, program=None):
		if value == True or type(value) == type(str) and value.lower() == "hardware":
			value = None
		else:
			value = "disabled"
		if program:
			path = 'HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\Direct3D' % program
		else:
			path = 'HKEY_USERS\\S-1-5-4\\Software\\Wine\\Direct3D'
		
		registry.set({path: {"VertexShaderMode": value}})
	
	def getPixelShader(self, program=None):
		if program:
			settings = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\Direct3D' % program, quiet=True)
		else:
			settings = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\Direct3D', quiet=True)
		if "PixelShaderMode" in settings:
			return False
		else:
			return True
	
	def setPixelShader(self, value, program=None):
		if value == True or type(value) == type(str) and value.lower() == "hardware":
			value = None
		else:
			value = "disabled"
		if program:
			path = 'HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\Direct3D' % program
		else:
			path = 'HKEY_USERS\\S-1-5-4\\Software\\Wine\\Direct3D'
		
		registry.set({path: {"PixelShaderMode": value}})
	
	def getFontAntialiasing(self):
		settings = registry.get('HKEY_CURRENT_USER\\Control Panel\\Desktop')
		return 'FontSmoothing' in settings and settings['FontSmoothing'] == "2"
	
	def setFontAntialiasing(self, value):
		if value:
			reg = {
				"FontSmoothing": 2,
				"FontSmoothingType": "dword:00000002",
				"FontSmoothingGamma": "dword:00000578",
				"FontSmoothingOrientation": "dword:00000001"
			}
		else:
			reg = {
				"FontSmoothing": None,
				"FontSmoothingType": None,
				"FontSmoothingGamma": None,
				"FontSmoothingOrientation": None
			}
			
		registry.set({'HKEY_CURRENT_USER\\Control Panel\\Desktop': reg})
	
	def check3DSupport(self):
		"""
			Check if the system has support for direct rendering
			Returns True or False
		"""
		glxinfo = subprocess.Popen(["glxinfo"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
		grep = subprocess.Popen(["grep", "direct rendering"], stdin=glxinfo.stdout, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
		result = grep.communicate()[0]
		if "y" in result.split(':')[1].lower():
			return True
		else:
			return False
	
	def getAntialiasingDisabled(self):
		settings = registry.get('HKEY_CURRENT_USER\\Software\\Wine\X11 Driver')
		if "ClientSideAntiAliasWithRender" in settings and "ClientSideAntiAliasWithCore" in settings \
		  and settings['ClientSideAntiAliasWithRender'].lower() == 'n' and settings['ClientSideAntiAliasWithCore'].lower() == 'n':
			return True
		else:
			return False
	
	def setAntialiasingDisabled(self, value):
		if value:
			value = "N"
		else:
			value = None
		
		registry.set({'HKEY_CURRENT_USER\\Software\\Wine\X11 Driver': {
			"ClientSideAntiAliasWithRender": value,
			"ClientSideAntiAliasWithCore": value
		}})
	
	def getVideoMemorySize(self):
		settings = registry.get('HKEY_CURRENT_USER\\Software\\Wine\\Direct3D')
		if 'VideoMemorySize' in settings:
			return settings['VideoMemorySize']
		else:
			return None
	
	def setVideoMemorySize(self, value):
		if value != None and type(value) != type(1):
			raise ValueError("type of value should be None or int")
		registry.set({'HKEY_CURRENT_USER\\Software\\Wine\\Direct3D': {
			'VideoMemorySize': value
		}})
	
	def getOffscreenRenderingMode(self):
		settings = registry.get('HKEY_CURRENT_USER\\Software\\Wine\\Direct3D')
		if 'OffscreenRenderingMode' in settings:
			return settings['OffscreenRenderingMode']
		else:
			if int(self.VERSION.replace('.','')) >= 1123:
				return 'fbo'
			else:
				return 'backbuffer'
	
	def setOffscreenRenderingMode(self, value):
		if value != None and value.lower() not in ['backbuffer', 'pbuffer', 'fbo']:
			raise ValueError("type of value should be None or int")
		registry.set({'HKEY_CURRENT_USER\\Software\\Wine\\Direct3D': {
			'OffscreenRenderingMode': value.lower()
		}})
