#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2008 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>

import _registry
import os

registry = _registry.Registry()

class Libraries:
	def __init__(self):
		self.libraryPackages = {
			"Microsoft Visual C++ 2008": ('msvcr90.dll', 'msvcp90.dll', 'msvcr90d.dll', 'msvcp90d.dll'),
			"Microsoft Visual C++ 2005": ('msvcr80.dll', 'msvcp80.dll', 'msvcr80d.dll', 'msvcp80d.dll'),
			"Microsoft Visual C++ 2003": ('msvcr71.dll', 'msvcp71.dll', 'msvcr71d.dll', 'msvcp71d.dll'),
			"Microsoft Visual C++ 2002": ('msvcr70.dll', 'msvcp70.dll', 'msvcr70d.dll', 'msvcp70d.dll'),
			"Microsoft Visual C++ 6.0": ('msvcrt.dll', 'msvcp60.dll', 'msvcrtd.dll', 'msvcp60d.dll'),
			"Microsoft Visual C++ 5.0": ('msvcrt.dll', 'msvcp50.dll', 'msvcrtd.dll', 'msvcp50d.dll'),
			"Microsoft Visual C++ 4.2": ('msvcrt.dll', 'msvcrtd.dll', 'msvcprt.lib', 'msvcprtd.lib'),
			"Windows Media Player": ('blackbox.dll', 'cewmdm.dll', 'drmclien.dll', 'drmstor.dll', 'drmv2clt.dll', 'laprxy.dll', 'mp43dmod.dll', 'mp4sdmod.dll', 'mpg4dmod.dll', 'msdmo.dll', 'msnetobj.dll', 'mspmsnsv.dll', 'mspmsp.dll', 'msscp.dll', 'mswmdm.dll', 'npdrmv2.dll', 'npwmsdrm.dll', 'qasf.dll', 'wdfapi.dll', 'wmadmod.dll', 'wmadmoe.dll', 'wmasf.dll', 'wmdmlog.dll', 'wmdmps.dll', 'wmdrmdev.dll', 'wmdrmnet.dll', 'wmidx.dll', 'wmnetmgr.dll', 'wmsdmod.dll', 'wmsdmoe2.dll', 'wmspdmod.dll', 'wmspdmoe.dll', 'wmvadvd.dll', 'wmvadve.dll', 'wmvcore.dll', 'wmvdmod.dll', 'wmvdmoe2.dll', 'wpd_ci.dll', 'wpdconns.dll', 'wpdmtp.dll', 'wpdmtpdr.dll', 'wpdmtpus.dll', 'wpdsp.dll', 'wpdtrace.dll')
		}
		self.builtinLibraries = "advapi32", "capi2032", "dbghelp", "ddraw", "gdi32", "glu32", "icmp", "gphoto2.ds", "iphlpapi", "kernel32", "mountmgr.sys", "mswsock", "ntdll", "ntoskrnl.exe", "opengl32", "sane.ds", "twain_32", "unicows", "user32", "vdmdbg", "w32skrnl", "winealsa.drv", "wineaudioio.drv", "wined3d", "winedos", "wineesd.drv", "winejack.drv", "winejoystick.drv", "winemp3.acm", "winenas.drv", "wineoss.drv", "wineps", "wineps.drv", "winex11.drv", "winmm", "wintab32", "wnaspi32", "wow32", "ws2_32", "wsock32"
	
	def list(self):
		for libpath in ["/usr/lib", "/usr/local/lib", "/usr/lib32", "/usr/local/lib32", "/usr/lib64", "/usr/local/lib64"]:
			if os.path.exists("%s/wine" % libpath):
				libs = [ i[:-7] for i in filter(lambda i: i.endswith(".dll.so"), os.listdir("%s/wine" % libpath)) ]
				break
		
		libraries = []
		for lib in libs:
			if lib not in self.builtinLibraries:
				libraries.append(lib)
		
		return sorted(libraries)
	
	def getOverrides(self):
		libs = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\DllOverrides', quiet=True)
		overridden = []
		for lib, setting in libs.iteritems():
			if not setting.startswith("builtin"):
				overridden.append(lib)
		return sorted(overridden)
	
	def setOverride(self, override, library):
		if library.endswith(".dll"):
			library = library[:-4]
		if override:
			registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\DllOverrides': {library: "native,builtin"}})
		else:
			registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\DllOverrides': {library: None}})
