#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2008 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>

import os
import util

import subprocess

import logging

class Registry:
	def __init__(self):
		self.registryCache = None
		self.logging = logging.getLogger("python-wine.programs")
		self.debug = self.logging.debug
		self.error = self.logging.error
	
	def __branch_to_dict(self, branch, dict):
		parents = []
		for path in filter(len, branch.split('\\')):
			pathname = 'dict%s' % ''.join([ '[\'%s\']' % i for i in parents ])
			if path not in eval(pathname) or type(eval(pathname)) != type({}):
				exec('%s[\'%s\'] = {}' % (pathname, path))
			parents.append(path)
		return dict
	
	def __set_branch_values(self, branch, dict, items):
		# Make sure the dict keys exist
		dict = self.__branch_to_dict(branch, dict)
		# Set the value
		pathname = 'dict%s' % ''.join([ '[\'%s\']' % i for i in filter(len, branch.split('\\')) ])
		for key, value in items.iteritems():
			try:
				exec ('%s[\'%s\'] = value' % (pathname, key.replace('\\', '\\\\').replace("'", '\\')))
			except ValueError:
				self.error("Error in registry! Pathname: %s\tKey: %s\tValue: %s" % (pathname, key, value))
		return dict
	
	def _writeRegistry(self, registrycontent):
		regfilename = util.tempname('vineyard-setregistry-', '.reg')
		regfile = open(regfilename, "w")
		regfile.write(registrycontent)
		regfile.close()
		returnvalue = subprocess.call(["regedit", regfilename], env=os.environ)
		#os.remove(regfilename)
		return returnvalue
	
	def __get_branch(self, branch=None, quiet=True):
		self._initWine()
		if branch == None:
			process = subprocess.Popen(["regedit", "/E", "-"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=os.environ)
			branch = ''
		else:
			process = subprocess.Popen(["regedit", "/E", "-", branch], stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=os.environ)
		registry = ''
		buffer = process.stdout.readline()
		while buffer != '':
			registry += buffer
			buffer = process.stdout.readline()
		#if process.returncode and not quiet:
		if registry == '' or registry.strip() == 'Success':
			if not quiet:
				self.error("Warning: Registry branch '%s' doesn't exist." % branch)
			return ''
		return registry
	
	def __get_branch_as_dict(self, branch=None):
		registry = self.__get_branch(branch)
		
		if not registry:
			return None
		
		registry_dict = {}
		
		path = None
		values = {}
		registrysplit = filter(len, [ i.strip() for i in registry.split("\r\n") ])+ ['[]']
		for linenr in range(len(registrysplit)):
			line = registrysplit[linenr]
			original_value = None
			if line.startswith('['):
				""" If line is section header """
				# Set the values of the last section, if any
				if path != None:
					registry_dict = self.__set_branch_values(path, registry_dict, values)
				# Get the section path and start a new section values dict
				path = ':'.join(line[1:].split(']')[:-1])
				values = {}
			elif line.startswith('"') or line.startswith('@'):
				""" If line is key/value """
				key = line.split('=')[0].replace('"','')
				value = "=".join(line.split('=')[1:]).strip()
				value_is_string = True
				value_type = value.split(':')[0].lower()
				""" If value is multi-line """
				if value.strip().endswith('\\'):
					value = "%s%s" % (value.strip()[:-1], registrysplit[linenr+1].strip())
					linenr += 2
					while value.strip().endswith('\\'):
						value = value[:-1] + registrysplit[linenr].strip()
						linenr += 1
					if value.endswith('\\'):
						value = value[:-1]
				""" See what type of value this is.
				    For more info, see http://technet.microsoft.com/en-us/library/bb727154.aspx
				"""
				if value_type.startswith('hex(2)'):
					""" Expanded String """
					try:
						new_value = util.hextoutf8(value)
						original_value = value
						value = new_value
					except UnicodeDecodeError:
						# This won't convert, it's probably an actual hex value, and not just stored as one
						#print "Error in path \"%s\", string:" % path, value
						value_is_string = False
						print "Couldn't read registry! This is a serious error, please report this error along with this text at https://bugs.launchpad.net/vineyard"
						print path, key, value_type, value
						exit(0)
				elif value_type.startswith('hex(7)'):
					""" Multi String """
					value_is_string = False
					try:
						value = filter(len, util.hextoutf8(value).split('\x00'))
					except TypeError:
						print "Couldn't read registry! This is a serious error, please report this error along with this text at https://bugs.launchpad.net/vineyard"
						print path, key, value_type, value
						exit(0)
				elif value_type.startswith('hex'):
					""" HEX Value """
					value_is_string = False
					value = value
				elif value_type.startswith('dword'):
					value_is_string = False
				elif value_type.startswith('null'):
					value_is_string = False
					value = None
				elif value_type.startswith('quote') or value.startswith('str') or value.startswith('str(2)'):
					value = value
				if value_is_string:
					if value.strip().startswith('"') and value.replace('\x00','').strip().endswith('"'):
						value = '"'.join(value.strip().split('"')[1:-1])
					
					try:
						#value = value.encode('utf-8').decode('string_escape').decode('utf-8').replace('\\\\', '\\').replace('\\\\', '\\').replace('\x00','')
						if type(value) == list:
							value = [ util.stringtoutf8(i) for i in value ]
						else:
							value = util.stringtoutf8(value)
					except UnicodeDecodeError:
						print "Couldn't read registry! This is a serious error, please report this error along with this text at https://bugs.launchpad.net/vineyard"
						print path, key, value_type, value.encode('hex')
						exit(0)
				values[key] = value
				if original_value:
					values['_%s' % key] = original_value
		return registry_dict
	
	def __branchpath_in_dict(self, dict, branch):
		try:
			self.__get_dict_from_branchpath(dict, branch)
			return True
		except:
			return False
	
	def __get_dict_from_branchpath(self, dict, branch):
		try:
			return eval('dict%s' % ''.join([ '[\'%s\']' % i for i in filter(len, branch.split('\\'))]))
		except NameError:
			return False
	
	def __set_dict_from_branchpath(self, dict, branch, value):
		exec('dict%s = value' % ''.join([ '[\'%s\']' % i for i in filter(len, branch.split('\\'))]))
		return dict
	
	def get(self, branch=None, findkey=None, quiet=True):
		if findkey:
			fullbranch = '%s\\%s' % (branch, findkey)
		else:
			fullbranch = branch
		
		reg = self.__get_branch_as_dict(branch)
		
		if reg == None:
			reg = {}
		else:
			reg = self.__get_dict_from_branchpath(reg, branch)
			
		if findkey:
			if findkey in reg:
				retvalue = reg[findkey]
			else:
				retvalue = None
		else:
			retvalue = reg
		
		return retvalue

	def set(self, branches):
		# Set registry values
		# Branches should be a dict similar to:
		# {
		#  'HKEY_CURRENT_USER\\Software\\Wine':
		#   {"Version": "winnt"},
		# 'HKEY_USERS\\S-1-5-4\\Control Panel\\Desktop':
		#   {"FontSmoothing": "2"}
		# }
		self._initWine()
		
		registry = "REGEDIT4\n"

		definedbranches = []
		for b in sorted(branches.keys()):
			branchwalk = b.split('\\')
			for i in range(len(branchwalk)-1):
				branch = "\n[%s\\%s]\n" % (branchwalk[0], '\\'.join(branchwalk[1:i+2]))
				if branch not in definedbranches:
					registry += branch
					definedbranches.append(branch)
			# Add the values
			for key,value in sorted(branches[b].iteritems()):
				if value == '-' or value == None:
					registry += "\"%s\"=-\n" % str(key).encode('utf-8').encode('string_escape')
				elif str(value).startswith('hex:') or str(value).startswith('dword:'):
					registry += "\"%s\"=%s\n" % (str(key).encode('utf-8').encode('string_escape'), str(value))
				else:
					try:
						registry += "\"%s\"=\"%s\"\n" % (str(key).encode('utf-8').encode('string_escape'), str(value).encode('utf-8').encode('string_escape').replace('"', '\\"'))
					except TypeError:
						print "Error:",key,value
		registry += "\n"
		self._writeRegistry(registry)
	
	def _initWine(self):
		self._wineprocess = subprocess.Popen("wineserver -p10", shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=os.environ)
